@@ -1,5 +1,5 @@
 /* paste - merge lines of files
-   Copyright (C) 1997-2016 Free Software Foundation, Inc.
+   Copyright (C) 1997-2021 Free Software Foundation, Inc.
    Copyright (C) 1984 David M. Ihnat
 
    This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Written by David Ihnat.  */
 
@@ -231,9 +231,8 @@ paste_parallel (size_t nfiles, char **fnamptr)
       bool somedone = false;
       char const *delimptr = delims;
       size_t delims_saved = 0;	/* Number of delims saved in 'delbuf'. */
-      size_t i;
 
-      for (i = 0; i < nfiles && files_open; i++)
+      for (size_t i = 0; i < nfiles && files_open; i++)
         {
           int chr IF_LINT ( = 0);	/* Input character. */
           int err IF_LINT ( = 0);	/* Input errno value.  */
@@ -267,16 +266,15 @@ paste_parallel (size_t nfiles, char **fnamptr)
                  If an EOF or error, close the file.  */
               if (fileptr[i])
                 {
-                  if (ferror (fileptr[i]))
-                    {
-                      error (0, err, "%s", quotef (fnamptr[i]));
-                      ok = false;
-                    }
+                  if (!ferror (fileptr[i]))
+                    err = 0;
                   if (fileptr[i] == stdin)
                     clearerr (fileptr[i]); /* Also clear EOF. */
-                  else if (fclose (fileptr[i]) == EOF)
+                  else if (fclose (fileptr[i]) == EOF && !err)
+                    err = errno;
+                  if (err)
                     {
-                      error (0, errno, "%s", quotef (fnamptr[i]));
+                      error (0, err, "%s", quotef (fnamptr[i]));
                       ok = false;
                     }
 
@@ -411,16 +409,15 @@ paste_serial (size_t nfiles, char **fnamptr)
       if (charold != line_delim)
         xputchar (line_delim);
 
-      if (ferror (fileptr))
-        {
-          error (0, saved_errno, "%s", quotef (*fnamptr));
-          ok = false;
-        }
+      if (!ferror (fileptr))
+        saved_errno = 0;
       if (is_stdin)
         clearerr (fileptr);	/* Also clear EOF. */
-      else if (fclose (fileptr) == EOF)
+      else if (fclose (fileptr) != 0 && !saved_errno)
+        saved_errno = errno;
+      if (saved_errno)
         {
-          error (0, errno, "%s", quotef (*fnamptr));
+          error (0, saved_errno, "%s", quotef (*fnamptr));
           ok = false;
         }
     }
